import yaml
from openpyxl import load_workbook
import pickle
import os
from typing import DefaultDict, List
from google_auth_oauthlib.flow import Flow, InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload, MediaIoBaseDownload
from google.auth.transport.requests import Request
import datetime

from collections import defaultdict

from digitalguide_reader.standardinteraktion.Interaktion import Interaktion


def Create_Service(client_secret_file, api_name, api_version, *scopes):
    print(client_secret_file, api_name, api_version, scopes, sep='-')
    CLIENT_SECRET_FILE = client_secret_file
    API_SERVICE_NAME = api_name
    API_VERSION = api_version
    SCOPES = [scope for scope in scopes[0]]
    print(SCOPES)

    cred = None

    pickle_file = f'token_{API_SERVICE_NAME}_{API_VERSION}.pickle'
    # print(pickle_file)

    if os.path.exists(pickle_file):
        with open(pickle_file, 'rb') as token:
            cred = pickle.load(token)

    if not cred or not cred.valid:
        if cred and cred.expired and cred.refresh_token:
            cred.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                CLIENT_SECRET_FILE, SCOPES)
            cred = flow.run_local_server()

        with open(pickle_file, 'wb') as token:
            pickle.dump(cred, token)

    try:
        service = build(API_SERVICE_NAME, API_VERSION, credentials=cred)
        print(API_SERVICE_NAME, 'service created successfully')
        return service
    except Exception as e:
        print('Unable to connect.')
        print(e)
        return None

def sheed_id2excel(service, sheet_id, excel_file):
    byteData = service.files().export_media(
        fileId=sheet_id,
        mimeType="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
    ).execute()

    with open(excel_file, "wb") as f:
        f.write(byteData)



def excel2dict(sheet, interaktion_spalte, poi_spalte, aktion_spalte, format_spalte, inhalt_spalte, max_row=None):

    excel_dict = defaultdict(
        lambda: defaultdict(
                list
            )
        )
    previous_interaktion=""
    previous_pio=""
    previous_aktion=""

    for row in sheet.iter_rows(min_row=2, max_row=max_row):
        if row[interaktion_spalte] or row[poi_spalte] or row[aktion_spalte] or row[format_spalte] or row[inhalt_spalte]:

            if row[interaktion_spalte].value == None:
                interaktion = previous_interaktion
            else:
                interaktion = row[interaktion_spalte].value
                previous_interaktion = interaktion

            if row[poi_spalte].value == None:
                poi = previous_pio
            else:
                poi = row[poi_spalte].value
                previous_pio = poi

            if row[aktion_spalte].value == None:
                aktion = previous_aktion
            else:
                aktion = row[aktion_spalte].value
                previous_aktion = aktion

            if row[format_spalte].value == None:
                ValueError("Empty Field in {}".format(row[format_spalte]))
            else:
                format = row[format_spalte].value

            if row[inhalt_spalte].value == None:
                ValueError("Empty Field in {}".format(row[inhalt_spalte]))
            else:
                inhalt = row[inhalt_spalte].value

            excel_dict[(poi, interaktion)][aktion].append((format, inhalt))
    
    return excel_dict


import re


def readTriggers(format_inhalt_list, action):
    triggers = []
    for format, inhalt in format_inhalt_list:
        if format == "Liste":
            trigger_dict = {}
            trigger_dict["handler"] = "CommandHandler"
            trigger_dict["command"] = inhalt
            trigger_dict["action"] = action
            triggers.append(trigger_dict)
            trigger_dict = {}
            trigger_dict["handler"] = "MessageHandler"
            trigger_dict["filter"] = "regex"
            trigger_dict["action"] = action
            if inhalt == "Weiter":
                trigger_dict["regex"] = "WEITER_PATTERN"
            elif inhalt == "Ja":
                trigger_dict["regex"] = "JA_PATTERN"
            elif inhalt == "Wohin":
                trigger_dict["regex"] = "WOHIN_PATTERN"
            elif inhalt == "Nein":
                trigger_dict["regex"] = "NEIN_PATTERN"
            elif inhalt == "Zurueck":
                trigger_dict["regex"] = "ZURUECK_PATTERN"
            else:
                trigger_dict["regex"] = inhalt
                print("List {} ist nicht bekannt!".format(inhalt))
            triggers.append(trigger_dict)
        if format == "Rasa":
            trigger_dict = {}
            trigger_dict["handler"] = "MessageHandler"
            trigger_dict["filter"] = "rasa"
            trigger_dict["intent"] = inhalt
            trigger_dict["action"] = action
            triggers.append(trigger_dict)
        if format == "Freitext":
            trigger_dict = {}
            trigger_dict["handler"] = "MessageHandler"
            trigger_dict["filter"] = "text"
            trigger_dict["action"] = action
            triggers.append(trigger_dict)
        if format == "Foto":
            trigger_dict = {}
            trigger_dict["handler"] = "MessageHandler"
            trigger_dict["filter"] = "photo"
            trigger_dict["action"] = action
            triggers.append(trigger_dict)
        if format == "Regex":
            trigger_dict = {}
            trigger_dict["handler"] = "MessageHandler"
            trigger_dict["filter"] = "regex"
            trigger_dict["regex"] = inhalt
            trigger_dict["action"] = action
            triggers.append(trigger_dict)
    return triggers


from collections import defaultdict
from digitalguide_reader.standardinteraktion.Interaktion import Interaktion

class Route:
    def __init__(self):
        self.interaktionen = []
        self.states = defaultdict(list)
        self.actions = defaultdict(list)

    def from_excel_dict(self, excel_dict, messanger="Telegram", asset_url=""):
        self.messanger = messanger
        for excel_dict_item in excel_dict.items():
        
            interaktion = Interaktion.from_excel_dict(excel_dict_item, messanger=messanger, asset_url=asset_url)
            
            self.interaktionen.append(interaktion)

        self.generate_states()
        self.generate_actions()

        self.validate()

    def validate(self):
        action_requirements = []
        for interaktion in self.interaktionen:
            action_requirements += interaktion.action_requirements

        for action_requirement in action_requirements:
            if not action_requirement in self.actions.keys():
                print("ERROR: Die erforderliche Aktion {} ist nicht definiert.".format(action_requirement))

    def generate_states(self):
        self.states = {}
        for interaktion in self.interaktionen:
            self.states = {**self.states, **interaktion.states}

        if self.messanger == "Telegram":
            self.states["NONE"]=[]
        if self.messanger == "WhatsApp":
            from configparser import ConfigParser

            config = ConfigParser()
            config.read("config.ini")

            self.states["START_START"]=[{"handler": "TypeHandler", "type": "Update", "action": config["bot"]["start_action"]}]

    def generate_actions(self):
        self.actions = {}
        for interaktion in self.interaktionen:
            self.actions = {**self.actions, **interaktion.actions}
        



from setuptools import find_packages, setup

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='digitalguide_reader',
    packages=find_packages(),
    version='0.0.8',
    description='A Python Library to read in a google sheet and turn it into states and actions',
    author='Soeren Etler',
    license='MIT',
    install_requires=["openpyxl",
                      "PyYAML",
                      "requests",
                      "gunicorn",
                      "google-api-python-client",
                      "google-auth-httplib2",
                      "google-auth-oauthlib",
                      "python-json-logger"],
    long_description=long_description,
    long_description_content_type='text/markdown'
)




import re
import os
import requests
import subprocess

def readActions(format_inhalt_list, messanger="Telegram", asset_url=""):
    actions = []
    for format, inhalt in format_inhalt_list:
        
        format_split = format.split(" - ")
        if len(format_split) == 2 and format_split[1] != messanger:
            continue
        elif len(format_split) == 2:
            format = format_split[0]

        action = {}

        if format == "Textnachricht":
            if messanger =="Telegram":
                inhalt = inhalt.replace(".", "\.")
                inhalt = inhalt.replace("(", "\(")
                inhalt = inhalt.replace(")", "\)")
                inhalt = inhalt.replace("!", "\!")
                inhalt = inhalt.replace("?", "\?")
                inhalt = inhalt.replace("-", "\-")
                inhalt = inhalt.replace("+", "\+")
                inhalt = inhalt.replace("#", "\#")

                action["parse_mode"] = "MarkdownV2"

            action["type"] = "message"
            action["text"] = inhalt

        elif format == "Interaktive Textnachricht":
            action["type"] = "message"

            text_match = re.search(r"Text: (?P<text>.*)", inhalt)
            if text_match:
                action["text"] = text_match.group('text')
            else:
                print("In der intaraktiven Nachricht ist kein Text hinterlegt: {}".format(inhalt))

            text_match = re.search(r"Footer: (?P<footer>.*)", inhalt)
            if text_match:
                footer = text_match.group('footer')
                if len(footer) > 60:
                    print("Error: Footer can not be longer 60 characters: {}".format(footer)) 
                action["footer"] = footer
            else:
                print("Info: In der intaraktiven Nachricht ist kein Footer hinterlegt: {}".format(inhalt))

            buttons_matches = re.findall(r"Button_(?P<id>.*): (?P<button_text>.*)", inhalt)
            if buttons_matches:
                buttons = []
                for buttons_match in buttons_matches:
                    button = {}
                    button["id"] = buttons_match[0]
                    button["text"] = buttons_match[1]
                    if len(button["text"]) > 20:
                        print("Error: Button can not be longer 20 characters: {}".format(button["text"])) 
                    buttons.append(button)
                action["reply_buttons"] = buttons
            else:
                print("Info: In der intaraktiven Nachricht sind keine Buttons hinterlegt: {}".format(inhalt))

        elif format == "Audionachricht":
            action["type"] = "audio"
            regex = r"Datei: (?P<file>.*)\nAnzeigename: (?P<name>.*)\nPerformer: (?P<performer>.*)"

            match = re.search(regex, inhalt)
            if match:
                action["url"] = os.path.join(asset_url, match.group('file'))
                if requests.get(action["url"]).status_code != 200:
                    print("Die angegebenen Datei ({}) konnte nicht unter der Asset URL ({}) gefunden werden.".format(
                    match.group('file'), asset_url))
                action["title"] = match.group('name')
                action["performer"] = match.group('performer')
            else:
                action["url"] = os.path.join(asset_url, "platzhalter.mp3")
                action["title"] = inhalt
                action["performer"] = "🤖"

        elif format == "Sprachnachricht":
            action["type"] = "voice"
            regex = r"Datei: (?P<file>.*)"

            match = re.search(regex, inhalt)
            if match:
                mp3_filepath = os.path.join("assets/", match.group('file'))
                ogg_filename = match.group('file').replace(".mp3", ".ogg")
                ogg_filepath =  os.path.join("assets/", ogg_filename)
                subprocess.run(["ffmpeg", "-loglevel", "error", '-i', mp3_filepath, "-b:a", " 64k", "-ac", "1", '-acodec', 'libopus', '-y', "-hide_banner", ogg_filepath])

                if messanger=="Telegram":
                    action["file"] = ogg_filepath
                else:
                    action["url"] = os.path.join(asset_url, match.group('file'))
            else:
                action["url"] = os.path.join(asset_url, "platzhalter.mp3")
                action["caption"] = inhalt

        elif format == "Foto":
            action["type"] = "photo"            
            file_match = re.search(r"Datei: (?P<file>.*)", inhalt)

            if file_match:
                if messanger=="Telegram":
                    action["file"] = os.path.join("assets", file_match.group('file'))
                else:
                    action["url"] = os.path.join(asset_url, file_match.group('file'))
                    if requests.get(action["url"]).status_code != 200:
                        print("Die angegebene Datei ({}) konnte nicht unter der Asset URL ({}) gefunden werden.".format(
                        file_match.group('file'), asset_url))
            else:
                action["url"] = os.path.join(asset_url, "platzhalter.png")

            caption_match = re.search(r"Anzeigename: (?P<name>.*)", inhalt)
            if caption_match:
                action["caption"] = caption_match.group('name')

        elif format == "GIF":
            action["type"] = "document"
            regex = r"Datei: (?P<file>.*)\nAnzeigename: (?P<name>.*)"

            match = re.search(regex, inhalt)
            if match:
                action["url"] = os.path.join(asset_url, match.group('file'))
                if requests.get(action["url"]).status_code != 200:
                    print("Die angegebene Datei ({}) konnte nicht unter der Asset URL ({}) gefunden werden.".format(
                    match.group('file'), asset_url))
                action["caption"] = match.group('name')
            else:
                action["url"] = os.path.join(asset_url, "platzhalter.png")
                action["caption"] = inhalt

        elif format == "GPS":
            action["type"] = "venue"

            action["longitude"] = re.search(
                r"L: (?P<long>.*)", inhalt).group('long')
            action["latitude"] = re.search(r"B: (?P<lat>.*)", inhalt).group('lat')
            action["title"] = re.search(
                r"Anzeigename: (?P<name>.*)", inhalt).group('name')
            action["address"] = re.search(
                r"Adresse: (?P<address>.*)", inhalt).group('address')

        elif format == "Video":
            action["type"] = "video"
            regex = r"Datei: (?P<file>.*)\nAnzeigename: (?P<caption>.*)"

            match = re.search(regex, inhalt)
            if match:
                action["url"] = os.path.join(asset_url, match.group('file'))
                if requests.get(action["url"]).status_code != 200:
                    print("Die angegebene Datei ({}) konnte nicht unter der Asset URL ({}) gefunden werden.".format(
                    match.group('file'), asset_url))
                action["caption"] = match.group('caption')

        elif format == "Sticker":
            regex = r"ID: (?P<id>.*)$"

            match = re.search(regex, inhalt)
            action["type"] = "sticker"
            if match:
                action["id"] = match.group('id')
            else:
                print("Die angegebenen Informationen im Feld ({}) stimmen nicht mit dem angegebenen Format ({}) überein.".format(
                    inhalt, format))
        elif format == "Kontextspeicherung":
            action["type"] = "function"
            action["func"] = "save_text_to_context"
            action["key"] = inhalt
        elif format == "Return":
            action["type"] = "return"
            action["state"] = inhalt
        elif format == "Formel":
            action["type"] = "function"
            action["func"] = re.search(
                r"function: (?P<function>.*)", inhalt).group('function')
            for line in inhalt.split("\n")[1:]:
                if len(line.split(": "))==2:
                    argument, value = line.split(": ")
                    action[argument] = value

        else:
            print("Das angegebene Format ({}) ist nicht bekannt".format(format))

        if action:
            actions.append(action)

    return actions
